/*
 * Copyright (C) 2000  Neil Deason
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
 * MA  02111-1307, USA.
 *
 * Author's Address
 *
 * Neil Deason
 * (ndeason@ubiquity.net)
 * Ubiquity Software Corporation
 * Ubiquity House
 * Langstone Park
 * Newport
 * South Wales
 * NP18 2LH
 */
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.FileNotFoundException;

import java.net.DatagramSocket;
import java.net.DatagramPacket;
import java.net.InetAddress;
import java.net.SocketException;
import java.net.UnknownHostException;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

import MessengerObserver;

/**
 * A simple class to send SIP test messages stored in text files
 * over UDP and, optionally, listen for any response. Can be run 
 * as a command line program or via a GUI.
 *
 * Usage:
 *     java Messenger [-listen] <local-host> <local-port> <remote-host> <remote-port> <message-file>
 *<PRE>
 * Options:
 *     [-listen]  listen for responses
 * Commands:
 *     <local-host>    the local host name/IP address to listen on
 *     <local-port>    the local port to listen on
 *     <remote-host>   the remore host name to send to  
 *     <remote-port>   the remote port to send to
 *     <message-file>  the filename of the message to send
 * </PRE>
 */
public class Messenger implements Runnable, MessengerObserver
{
	private DatagramSocket m_DatagramSocket;
	private boolean m_Listening;
	private Thread m_ListeningThread;
	private MessengerObserver m_Observer;

	// set to true to see debug statements
	private boolean m_Debugging = false; 

	/**
	 * Construct a new Messenger.
	 */ 
	public Messenger()
	{
		System.out.println("SIP Messenger v1.1 (C) 2000 Neil Deason");
	}

	/**
	 * Set the Messenger's observer for notification of any
	 * incoming messages.
	 * @param a_Observer an implementer of the MessengerObserver interface.
	 */
	public void setObserver(MessengerObserver a_Observer)
	{
		m_Observer = a_Observer;
	}

	/**
	 * Open a Datagram Socket.
	 * @param a_Host the host name to open the Datagram Socket on.
	 * @param a_Port the port to open the Datagram Socket on.
	 */
	public boolean openDatagramSocket(String a_Host, int a_Port)
	{
		try
		{
			printDebug("Getting inetAddress");
			InetAddress inetAddress = InetAddress.getByName(a_Host);
			printDebug("Got inetAddress");
			m_DatagramSocket = new DatagramSocket(a_Port, InetAddress.getByName(a_Host));
			printDebug("Opened DatagramSocket on " + a_Host + ":" + a_Port);
			// if we have an observer start listening for responses
			if( null != m_Observer )
			{
				m_Listening = true;
				m_ListeningThread = new Thread(this);
				m_ListeningThread.start();
				printDebug("Started listener thread");
			}

			return true;
		}
		catch(IOException e)
		{
			System.out.println("Error opening Datagram Socket " + e);
			return false;
		}
	}

	/**
	 * Close the Datagram Socket.
	 */
	public void closeDatagramSocket()
	{
		if ( null != m_DatagramSocket )
		{
			m_DatagramSocket.close();
			m_DatagramSocket = null;
		}
	}

	/**
	 * Send a message.
	 * @param a_Address the host name to send the message to.
	 * @param a_Port the port to send the message to.
	 * @param a_FileName the filename of the message to send.
	 */
	public String send(String a_Address, int a_Port, String a_FileName)
	{
		String result;

		try
		{
			BufferedInputStream buffer = 
				new BufferedInputStream( new FileInputStream(a_FileName) );
			try 
			{
				int length = buffer.available();
				byte[] buf = new byte[length];
				buffer.read(buf, 0, length);
				DatagramPacket packet = new DatagramPacket(buf, 
														   length, 
														   InetAddress.getByName(a_Address),
														   a_Port);
				result = new String( packet.getData() );
				System.out.println("Preparing to send buffer:");
				System.out.println(result);
				m_DatagramSocket.send(packet);
			}
			catch(IOException e)
			{
				result = "Error sending datagram packet " + e;
			}
		}
		catch(FileNotFoundException e)
		{
			result = ("Error opening message file " + e);
		}

		
		return result;
	}

	/**
	 * A thread to listen for incoming messages and notify the Messenger's 
	 * Observer.
	 */
	public void run()
	{
		int packetSize = 1500;
		while (m_Listening)
		{
			DatagramPacket packet = new DatagramPacket(new byte[packetSize], packetSize);
			try
			{
				m_DatagramSocket.receive(packet);
				if ( null != m_Observer )
				{
					printDebug("Length of packet received " + packet.getLength());
					m_Observer.notify(new String( packet.getData(), 0, packet.getLength()) );
				}
				packet.setLength(packetSize);
			}
			catch(IOException e)
			{
				System.out.println("Error receiving datagram packet " + e);
			}

			try
			{
				Thread.sleep(250);
			}
			catch(InterruptedException e)
			{
			}
		}

	}

	/**
	 *  Implementation of MessengerObserver interface.
	 */
	public void notify(String response)
	{
		System.out.println("Received packet:");
		System.out.println(response);
	}

	/**
	 * Parse the command line arguments.
	 */
	private static String[] parseArguments(HashSet a_Switches,
									   String[] a_Arguments)
	{
		String[] result = new String[5];

		List args = Arrays.asList(a_Arguments);
		Iterator iArg = args.iterator();
		while (iArg.hasNext())
		{
			String arg = (String)iArg.next();
			if (arg.startsWith("-"))
			{
				a_Switches.add(arg.substring(1));
			}
			else
			{ // do some very primitive validation of the args
				if ( null == result[0])
				{
					// local host hasn't been seen yet
					result[0] = arg;
				}
				else if (null == result[1])
				{
					// local port hasn't been seen yet
					result[1] = arg;
				}
				else if (null == result[2])
				{
					// remote host hasn't been seen yet
					result[2] = arg;
				}
				else if (null == result[3])
				{
					// remote port hasn't been seen yet
					result[3] = arg;
				}
				else if ( null == result[4])
				{
					// message file hasn't been seen yet
					result[4] = arg;
				}
				else
				{
					// Oops!  Too Many Arguments specified
					displayHelp();
				}
			}
		}

		// Make sure that we have everything
		for( int i = 0; i < 5 ; i++)
		{
			if (null == result[i])
			{
				displayHelp();
			}
		}
		
		return result;
	}

	/**
	 * Print a usage summary.
	 */
	private static void displayHelp()
	{
		System.out.println(
			"Usage: Messenger [-listen] <local-host> <local-port> <remote-host> <remote-port> <message-file>\n");

		System.exit(0);
	}

	/**
	 * Pring debug statements, if debugging has been turned on.
	 */
	private void printDebug(String statement)
	{
		if( m_Debugging )
		{
			System.out.println(statement);
		}
	}

	public static void main(String[] args)
	{
		Messenger messenger = new Messenger();
		HashSet switches = new HashSet();
		String[] commands = parseArguments(switches, args);
		if( switches.contains("listen"))
		{
			System.out.println("Listening for responses");
			// set Messenger as it's own observer
			messenger.setObserver(messenger);
		}
		try
		{
			messenger.openDatagramSocket(commands[0], Integer.parseInt(commands[1]) );
			String result = messenger.send(commands[2], Integer.parseInt(commands[3]), commands[4]);
			if( null != result)
			{
				System.out.println(result);
			}
		}
		catch(NumberFormatException nfe)
		{
			displayHelp();
		}
	}
}
