/*
 * Copyright (C) 2000  Neil Deason
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
 * MA  02111-1307, USA.
 *
 * Author's Address
 *
 * Neil Deason
 * (ndeason@ubiquity.net)
 * Ubiquity Software Corporation
 * Ubiquity House
 * Langstone Park
 * Newport
 * South Wales
 * NP18 2LH
 */
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.io.*;
import java.util.ListIterator;

import Messenger;
import MessengerObserver;

/**
 * A GUI for a SIP test messenger application.
 */
public class MessengerGui extends JFrame implements MessengerObserver
{
	private final static String DEFAULT_PORT = "5060";
	private final static String COMMENT = "  Status: ";
	private final static String SPACE = "   ";

	JTextField m_LocalPortField;
	JTextField m_LocalHostField;
	JTextField m_RemoteHostField;
	JTextField m_RemotePortField;
	JTextField m_FileNameField;

	JLabel m_StatusLabel;

	JTextArea m_IncomingTextArea;
	JTextArea m_OutgoingTextArea;

	JButton m_SendButton;

	String m_FileName;
	Messenger m_Messenger;
	String m_LocalHost;
	int m_LocalPort;
	String m_RemoteHost;
	int m_RemotePort;
	

	boolean m_Listening;

	/**
	 * Construct a new MessengerGui.
	 */
	public MessengerGui()
	{
		m_Messenger = new Messenger();
		m_Messenger.setObserver(this);
		initGui();
	}

	/**
	 * Implementation of the MessengerObserver interface
	 */
	public void notify(String incoming)
	{
		// update Incoming buffer with received message
		m_IncomingTextArea.append("\n");
		m_IncomingTextArea.append(incoming);
		m_IncomingTextArea.append("\n");
		// scroll to bottom of new text
		m_IncomingTextArea.setCaretPosition(m_IncomingTextArea.getDocument().getLength());
	}

	/**
	 * Intialise the GUI components.
	 */
	private void initGui()
	{
		UIManager.put("ToolTip.background", Color.white);
		setTitle("SIP Messenger GUI running on " + getAddress() 
				 + "     v1.1 (C) 2000 Neil Deason");
		// local port components
		JPanel localPortPanel = new JPanel();
		m_LocalPortField = new JTextField(5);
		m_LocalPortField.setText(DEFAULT_PORT);
		m_LocalPortField.setFont(new Font("Helvetica", Font.PLAIN, 12));
		m_LocalPortField.setToolTipText("The Port Number to Listen on");
		m_LocalPortField.setSelectedTextColor(Color.blue);
		m_LocalPortField.setSelectionColor(Color.lightGray);
		JLabel localPortLabel = new JLabel("Port: ");
		localPortLabel.setForeground(new Color(13, 97, 200));
		localPortLabel.setFont(new Font("Helvetica", Font.BOLD, 12));
		localPortPanel.add(localPortLabel);
		localPortPanel.add(m_LocalPortField);

		// local host components
		JPanel localHostPanel = new JPanel();
		m_LocalHostField = new JTextField(10);
		m_LocalHostField.setToolTipText("The Host to Listen on");
		m_LocalHostField.setFont(new Font("Helvetica", Font.PLAIN, 12));
		m_LocalHostField.setSelectedTextColor(Color.blue);
		m_LocalHostField.setSelectionColor(Color.lightGray);
		JLabel localHostLabel = new JLabel("Host: ");
		localHostLabel.setForeground(new Color(13, 97, 200));
		localHostLabel.setFont(new Font("Helvetica", Font.BOLD, 12));
		localHostPanel.add(localHostLabel);
		localHostPanel.add(m_LocalHostField);

		// group local host and local port info together
		JPanel localAddressPanel = new JPanel();
		localAddressPanel.add(localHostPanel);
		localAddressPanel.add(localPortPanel);

   		// the Listen button
		JPanel listenButtonPanel = new JPanel();
		JButton listenButton = new JButton(SPACE+"LISTEN"+SPACE);
		listenButton.setBorder(new SoftBevelBorder(BevelBorder.RAISED));
		listenButton.setToolTipText("Listen for Incoming Messages");
		listenButton.setFont(new Font("Helvetica", Font.BOLD, 12));
		listenButton.addActionListener(new ListenActionListener());
		listenButton.registerKeyboardAction(new ListenActionListener(), 
									  KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), 
									  JComponent.WHEN_FOCUSED);
		listenButtonPanel.add(listenButton);

		// create a panel containing the local address, 
		// and listen button components
		JPanel listenPanel = new JPanel();
		listenPanel.setLayout(new BorderLayout());
		listenPanel.add("North", localAddressPanel);
		listenPanel.add(listenButtonPanel);
		listenPanel.setBorder(BorderFactory.createEtchedBorder());
		
		// the remote host components
		JPanel remoteHostPanel = new JPanel();	
		m_RemoteHostField = new JTextField(10);
		m_RemoteHostField.setToolTipText("The Host to Connect to");
		m_RemoteHostField.setFont(new Font("Helvetica", Font.PLAIN, 12));
		m_RemoteHostField.setSelectedTextColor(Color.blue);
		m_RemoteHostField.setSelectionColor(Color.lightGray);
		JLabel remoteHostLabel = new JLabel("Host: ");
		remoteHostLabel.setForeground(new Color(13, 97, 200));
		remoteHostLabel.setFont(new Font("Helvetica", Font.BOLD, 12));
		remoteHostPanel.add(remoteHostLabel);
		remoteHostPanel.add(m_RemoteHostField);	

		// the remote port components
		JPanel remotePortPanel = new JPanel();	
		m_RemotePortField = new JTextField(5);
		m_RemotePortField.setToolTipText("The Port Number to Connect to");
		m_RemotePortField.setFont(new Font("Helvetica", Font.PLAIN, 12));
		m_RemotePortField.setSelectedTextColor(Color.blue);
		m_RemotePortField.setSelectionColor(Color.lightGray);
		JLabel remotePortLabel = new JLabel("Port: ");
		remotePortLabel.setForeground(new Color(13, 97, 200));
		remotePortLabel.setFont(new Font("Helvetica", Font.BOLD, 12));
		remotePortPanel.add(remotePortLabel);
		remotePortPanel.add(m_RemotePortField);

		// group remote host and port together
		JPanel remoteAddressPanel = new JPanel();
		remoteAddressPanel.add(remoteHostPanel);
		remoteAddressPanel.add(remotePortPanel);

		// the filename components
		JPanel fileNamePanel = new JPanel();
		m_FileNameField = new JTextField(10);
		m_FileNameField.setToolTipText("Fliename of the SIP Message to Send");
		m_FileNameField.setFont(new Font("Helvetica", Font.PLAIN, 12));
		m_FileNameField.setSelectedTextColor(Color.blue);
		m_FileNameField.setSelectionColor(Color.lightGray);
		JLabel fileNameLabel = new JLabel("File Name: ");
		fileNameLabel.setForeground(new Color(13, 97, 200));
		fileNameLabel.setFont(new Font("Helvetica", Font.BOLD, 12));
		fileNamePanel.add(fileNameLabel);
		fileNamePanel.add(m_FileNameField);

		// the Send button
		m_SendButton = new JButton(SPACE+"SEND"+SPACE);
		m_SendButton.setBorder(new SoftBevelBorder(BevelBorder.RAISED));
		m_SendButton.setToolTipText("Send a Message");
		m_SendButton.setFont(new Font("Helvetica", Font.ITALIC|Font.BOLD, 12));
		m_SendButton.addActionListener(new SendActionListener());
		m_SendButton.registerKeyboardAction(new SendActionListener(), 
										  KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), 
										  JComponent.WHEN_FOCUSED);
		m_SendButton.setEnabled(false);
		
		// group the filename fields and send button together
		JPanel sendButtonPanel = new JPanel();
		sendButtonPanel.add(fileNamePanel);
		sendButtonPanel.add(m_SendButton);

		// create a panel to contain the remote address 
		// components and the Send button
		JPanel sendPanel = new JPanel();
		sendPanel.setLayout(new BorderLayout());
		sendPanel.add("North", remoteAddressPanel);
		sendPanel.add(sendButtonPanel);
		sendPanel.setBorder(BorderFactory.createEtchedBorder());

		// panel for listen/send buttons + their assoc components
		JPanel configPanel = new JPanel();
		configPanel.setLayout(new GridLayout(1,3));
		configPanel.add(listenPanel);
		configPanel.add(sendPanel);

		// create labels for incoming/outgoing message buffers
		JPanel buffersLabelPanel = new JPanel();
		JLabel out = new JLabel("Outgoing Message", JLabel.CENTER);
		out.setForeground(new Color(13, 97, 200));
		out.setFont(new Font("Helvetica", Font.BOLD, 12));
		JLabel in = new JLabel("Incoming Message", JLabel.CENTER);
		in.setForeground(new Color(13, 97, 200));	
		in.setFont(new Font("Helvetica", Font.BOLD, 12));
		buffersLabelPanel.setLayout(new GridLayout(1,2));
		buffersLabelPanel.add(out);
		buffersLabelPanel.add(in);
		buffersLabelPanel.setBorder(BorderFactory.createEtchedBorder());

		// group the buffer labels below the main config panel
		JPanel lowerHeaderPanel = new JPanel();
		lowerHeaderPanel.setLayout(new BorderLayout());
		lowerHeaderPanel.add("North", configPanel);
		lowerHeaderPanel.add("South", buffersLabelPanel);

		// create the top panel with a Status label, the config
		// components and the buffer labels
		JPanel headerPanel = new JPanel();
		headerPanel.setLayout(new BorderLayout());
		m_StatusLabel = new JLabel(COMMENT);		
		m_StatusLabel.setFont(new Font("Helvetica", Font.BOLD, 14));
		m_StatusLabel.setForeground(new Color(13, 97, 200));
		headerPanel.add("North", m_StatusLabel);
		headerPanel.add(lowerHeaderPanel);

		// create Incoming/Outgoing message buffers
		JPanel messagesPanel=new JPanel();
		messagesPanel.setLayout(new GridLayout(1, 2));
	
		m_OutgoingTextArea = new JTextArea(25,45);
		m_OutgoingTextArea.setToolTipText("Messages Sent");
		m_OutgoingTextArea.setEditable(false);
		m_OutgoingTextArea.setFont(new Font("Helvetica", Font.PLAIN, 13));
		m_OutgoingTextArea.setSelectedTextColor(Color.blue);
		m_OutgoingTextArea.setSelectionColor(Color.lightGray);

		m_IncomingTextArea = new JTextArea(25, 45);
		m_IncomingTextArea.setToolTipText("Messages Received");
		m_IncomingTextArea.setEditable(false);
		m_IncomingTextArea.setFont(new Font("Helvetica", Font.PLAIN, 13));
		m_IncomingTextArea.setSelectedTextColor(Color.blue);
		m_IncomingTextArea.setSelectionColor(Color.lightGray);

		// create scroll bars for text areas
		JScrollPane incomingScroller = new JScrollPane(m_IncomingTextArea);
		incomingScroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		incomingScroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		JScrollPane outgoingScroller = new JScrollPane(m_OutgoingTextArea);
		outgoingScroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		outgoingScroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);

		messagesPanel.add(outgoingScroller);
		messagesPanel.add(incomingScroller);
		messagesPanel.setBorder(BorderFactory.createEtchedBorder());

		// create Quit and Clear buttons.
		JPanel guiControlPanel = new JPanel();
		JButton quitButton = new JButton(SPACE+"QUIT"+SPACE);
		quitButton.setBorder(new SoftBevelBorder(BevelBorder.RAISED));
		quitButton.setToolTipText("Ends the Session");
		quitButton.setFont(new Font("Helvetica", Font.BOLD, 13));
		quitButton.addActionListener(new QuitActionListener());
		quitButton.registerKeyboardAction(new QuitActionListener(), 
										  KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), 
										  JComponent.WHEN_FOCUSED);
		JButton resetButton = new JButton(SPACE+"CLEAR TEXT"+SPACE);
		resetButton.setBorder(new SoftBevelBorder(BevelBorder.RAISED));
		resetButton.setToolTipText("Resets the Message Text Areas");
		resetButton.setFont(new Font("Helvetica", Font.BOLD, 13));
		resetButton.addActionListener(new ResetActionListener());
		resetButton.registerKeyboardAction(new ResetActionListener(), 
										   KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), 
										   JComponent.WHEN_FOCUSED);
		guiControlPanel.add(quitButton);
		guiControlPanel.add(resetButton);

		// add all the components to GUI
		Container mainContainer = this.getContentPane();
		mainContainer.setLayout(new BorderLayout());
		mainContainer.add("North",headerPanel);
		mainContainer.add("Center", messagesPanel);
		mainContainer.add("South", guiControlPanel);
		this.addWindowListener(new WindowHandler());
	}

	/**
	 * Perform the action for the Listen button.
	 */
	private void doListen()
	{
		String port = m_LocalPortField.getText().trim();
		String host = m_LocalHostField.getText().trim();
		String statusResult = COMMENT;

		if(host.length() == 0)
		{
			statusResult += "No Host Name Entered"; 
		}
		else if(port.length() == 0)
		{
			statusResult += "Listening Port is Empty";
		}
		else
		{
		   
			int iPort = Integer.parseInt(port);
			if(!m_Listening || !host.equals(m_LocalHost) || iPort != m_LocalPort)
			{
				m_Messenger.closeDatagramSocket();
				m_Listening = m_Messenger.openDatagramSocket(host, iPort);
				if( m_Listening )
				{
					m_LocalHost = host;
					m_LocalPort = iPort;
					statusResult += ( ("Listening on host " + host + " port " + iPort) );
					m_SendButton.setEnabled(true);
				}
				else
				{
					m_Listening = false;
					statusResult += ( ("Could Not Listen on host " + host + " port " + iPort) );
					m_SendButton.setEnabled(false);
				}
			}
		}
		m_StatusLabel.setText(statusResult);
	}


	/**
	 * Perform the action for the Send button.
	 */
	private void doSend()
	{
		String host = m_RemoteHostField.getText().trim();
		String port = m_RemotePortField.getText().trim();
		m_FileName = m_FileNameField.getText().trim();
		String statusResult = COMMENT;

		if(host.length() == 0)
		{
			statusResult += "No Host Name Entered";
		}
		else if(port.length() == 0)
		{
			statusResult += "No Port Number Entered";
		}
		else if(m_FileName.length() == 0)
		{
			statusResult += "No File Name Entered";
		}
		else
		{
			try
			{
				int iPort = Integer.parseInt(port);
				m_OutgoingTextArea.append("\n");
				m_OutgoingTextArea.append( m_Messenger.send(host, iPort, m_FileName) );
				m_OutgoingTextArea.append("\n");
				statusResult += ("sent " + m_FileName + " to " + host + ":" + port);
			}
			catch(NumberFormatException e)
			{
				statusResult += "Invalid Port Number Entered" + e;
			}
		}
		m_StatusLabel.setText(statusResult);
	}

	/**
	 * Perform the action for the Quit button.
	 */
	private void doQuit()
	{
		this.dispose();
		System.exit(0);
	}

	/**
	 * Perform the action for the Clear button.
	 */
	private void doReset()
	{	
		m_IncomingTextArea.setText("");
		m_OutgoingTextArea.setText("");
	}

	/**
	 * Get the address of the local host.
	 */
	private String getAddress()
	{
		String address="";
		try
		{
			InetAddress ipAddress = InetAddress.getLocalHost();
			address = ipAddress.getHostName();
		}
		catch(UnknownHostException uhe){}
		return address;
	}

	// Inner Classes to act as event listeners for us
	private class ListenActionListener implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			doListen();
		}
	}
	
	private class SendActionListener implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			doSend();
		}
	}
	
	private class QuitActionListener implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			doQuit();
		}
	}

	private class ResetActionListener implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{			doReset();
		}
	}

	private class WindowHandler extends WindowAdapter
	{
		public void windowClosed(WindowEvent e) 
		{
			System.exit(0);
		}
		   
		public void windowClosing(WindowEvent e)
		{
			setVisible(false);
			dispose();
		}
	}

	public static void main(String[] args)
	{
		MessengerGui gui = new MessengerGui();
		gui.pack();
		gui.setVisible(true);	
	}
}
